; SETSPEED - Set CPU Speed on NuXT motherboard. Using the following parameters:
;
; SETSPEED 4 (4.77MHz - Slow)
; SETSPEED 7 (7.16MHz - Medium)
; SETSPEED 9 (9.55Mhz - Fast)
;
;-------------------------------------------------------------------------------

BIOS_Data_Area  EQU     40H     ;BIOS Data Area
Equip_List      EQU     10H                     ; Equpment list
Equip_FPU       EQU     0000000000000010B       ; FPU installed
FE_Config_Reg   EQU     63H     ; Chipset configuration register
FE_Par_Disable  EQU     01H     ; Disable memory parity checking
FE_Clk_716MHz   EQU     40H     ; FE2010A 7.16 MHz CPU clock frequency
FE_Clk_955MHz   EQU     80H     ; FE2010A 9.55 MHz CPU clock frequency

Code    SEGMENT PARA    PUBLIC  'CODE'

        ASSUME  CS:Code, DS:Code, ES:Code, SS:Code

        ORG     100H            ;COM files start at 100H

Begin:
        JMP     Main            ;Skip past data

InfoMsg         DB      'SETSPEED - NuXT motherboard CPU speed setting program', 13, 10
                DB      13, 10
                DB      'Use the following parameters:', 13, 10
                DB      13, 10
                DB      'SETSPEED 4 - Set CPU to 4.77MHz', 13, 10
                DB      'SETSPEED 7 - Set CPU to 7.16MHz', 13, 10
                DB      'SETSPEED 9 - Set CPU to 9.55MHz', 13, 10, '$'

CPU477Msg       DB      'CPU speed set to 4.77Mhz', 13, 10, '$'
CPU716Msg       DB      'CPU speed set to 7.16Mhz', 13, 10, '$'
CPU955Msg       DB      'CPU speed set to 9.55Mhz', 13, 10, '$'

;*******************************************************************************

Main:
        CALL    GetParm         ;Get parameter number from command line
        CMP     BL,0            ;Check if command line is invalid
        JNE     SetSpeed        ; ..
        MOV     AH,9            ;Print info message and exit
        MOV     DX,OFFSET InfoMsg ; ..
        INT     21H             ; ..
        JMP     EXIT            ; ..
SetSpeed:
        PUSH    DS              ;Get BIOS Data Area Equipment list lower byte
        MOV     AX,40H          ; ..
        MOV     DS,AX           ; ..
        MOV     AL,[Equip_List] ; ..
        POP     DS              ; ..
        CMP     BL,'4'          ;Check for 4.77MHz
        JE      CPU477MHz       ; ..
        CMP     BL,'7'          ;Check for 7.16MHz
        JE      CPU716MHz       ; ..
        CMP     BL,'9'          ;Check for 9.55MHz
        JE      CPU955MHz       ; ..
CPU477MHz:
        AND     AL,Equip_FPU            ;Isolate FPU installed bit
        OR      AL,FE_Par_Disable       ;Disable memory parity checking
        OUT     FE_Config_Reg,AL        ;Clear bits 7-5 - 4.77 MHz CPU clock
        MOV     AH,9                    ;Print speed and exit
        MOV     DX,OFFSET CPU477Msg     ; ..
        INT     21H                     ; ..
        JMP     EXIT                    ; ..
CPU716MHz:
        AND     AL,Equip_FPU            ;Isolate FPU installed bit
        OR      AL,FE_Par_Disable       ;Disable memory parity checking
        OR      AL,FE_Clk_716MHZ        ;Set bit 6 - 7.16 MHz CPU clock
        OUT     FE_Config_Reg,AL        ;Set the speed
        MOV     AH,9                    ;Print speed and exit
        MOV     DX,OFFSET CPU716Msg     ; ..
        INT     21H                     ; ..
        JMP     EXIT                    ; ..
CPU955MHz:
        AND     AL,Equip_FPU            ;Isolate FPU installed bit
        OR      AL,FE_Par_Disable       ;Disable memory parity checking
        OR      AL,FE_Clk_955MHZ        ;Set bit 7 - 7.16 MHz CPU clock
        OUT     FE_Config_Reg,AL        ;Set the speed
        MOV     AH,9                    ;Print speed and exit
        MOV     DX,OFFSET CPU955Msg     ; ..
        INT     21H                     ; ..
EXIT:
        MOV     AX,4C00H        ;Exit to DOS
        INT     21H             ; ..
;-------------------------------------------------------------------------------

;
;Get parameter from command line
;
GetParm         PROC    NEAR

        MOV     BX,80H          ;Point to beginng of command line
        MOV     AL,[BX]         ;See if there is anything on command line
        OR      AL,AL           ; ..
        JZ      GetParm2        ;No, exit
        MOV     AL,[BX+2]       ;Get the parameter
        CMP     AL,'4'          ;Check for 4
        JE      GetParm1        ; ..
        CMP     AL,'7'          ;Check for 7
        JE      GetParm1        ; ..
        CMP     AL,'9'          ;Check for 9
        JE      GetParm1        ; ..
        JMP     GetParm2        ;No, exit

GetParm1:
        MOV     BL,AL           ;Save the selected speed
        RET                     ; ..
GetParm2:
        MOV     BL,0            ;Indicate parameter is invalid
        RET

GetParm         ENDP

Code    ENDS

;*******************************************************************************

        END     Begin
